//--------------------------------------------------------------
// File     : stm32_ub_icfrq_tim2.c
// Datum    : 08.07.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : GPIO, TIM, MISC
// Funktion : Input-Capture (Frq-Messung) per Timer2
//
// Hinweis  : mgliche Pinbelegungen
//            CH1 : [PA0, PA5]
//            CH2 : [PA1, PB3]
//            CH3 : [PA2, PB10]
//            CH4 : [PA3, PB11]
//
// Settings : Timer2-Frq steht auf 1MHz = 1us (siehe H-File)
//            Ein Messwert von 1     => 1us     = 1MHz
//            Ein Messwert von 10    => 10us    = 100kHz
//            Ein Messwert von 65535 => 65,53ms = 15 Hz
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_icfrq_tim2.h"



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_ICFRQ_InitIO(void);
void P_ICFRQ_InitTIM(void); 
void P_ICFRQ_InitNVIC(void);


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
ICFRQ_TIM2_Var_t ICFRQ_TIM2_Var[4];


//--------------------------------------------------------------
// Definition aller InputCapture Pins (zur Frequenzmessung)
// Reihenfolge wie bei ICFRQ_TIM2_NAME_t
//
// Channel : [1...4]
//--------------------------------------------------------------
ICFRQ_TIM2_t ICFRQ_TIM2[] = {
  // Name       ,Channel, PORT , PIN       , CLOCK               , Source
  {ICFRQ_T2_PB11,4      ,GPIOB ,GPIO_Pin_11,RCC_AHB1Periph_GPIOB ,GPIO_PinSource11},
  {ICFRQ_T2_PA2 ,3      ,GPIOA ,GPIO_Pin_2 ,RCC_AHB1Periph_GPIOA ,GPIO_PinSource2},
};



//--------------------------------------------------------------
// init und start vom InputCapture (Frq-Messung) mit Timer2
//-------------------------------------------------------------- 
void UB_ICFRQ_TIM2_Init(void)
{
  // init aller Variabeln
  ICFRQ_TIM2_Var[0].flanke_nr=0;
  ICFRQ_TIM2_Var[0].wert0=0;
  ICFRQ_TIM2_Var[0].wert1=0;
  ICFRQ_TIM2_Var[0].wert=0;

  ICFRQ_TIM2_Var[1].flanke_nr=0;
  ICFRQ_TIM2_Var[1].wert0=0;
  ICFRQ_TIM2_Var[1].wert1=0;
  ICFRQ_TIM2_Var[1].wert=0;

  ICFRQ_TIM2_Var[2].flanke_nr=0;
  ICFRQ_TIM2_Var[2].wert0=0;
  ICFRQ_TIM2_Var[2].wert1=0;
  ICFRQ_TIM2_Var[2].wert=0;

  ICFRQ_TIM2_Var[3].flanke_nr=0;
  ICFRQ_TIM2_Var[3].wert0=0;
  ICFRQ_TIM2_Var[3].wert1=0;
  ICFRQ_TIM2_Var[3].wert=0;

  // init der Funktionen
  P_ICFRQ_InitIO();
  P_ICFRQ_InitTIM();
  P_ICFRQ_InitNVIC();
}


//--------------------------------------------------------------
// auslesen vom Input-Capture Wert (Frq) von einem Kanal
//
// Return_Wert : 0         => Messwert nicht gltig
//               1...65535 => Messwert
// Formeln :
//  Mess_Frq = T2_Frq / Messwert
//  Messwert = T2_Frq / Mess_Frq
//--------------------------------------------------------------
uint32_t UB_ICFRQ_TIM2_ReadFRQ(ICFRQ_TIM2_NAME_t vname)
{
  uint32_t ret_wert=0;

  if(ICFRQ_TIM2[vname].CHANNEL==1) {   
    ret_wert=ICFRQ_TIM2_Var[0].wert;
    ICFRQ_TIM2_Var[0].wert=0;
  }  
  if(ICFRQ_TIM2[vname].CHANNEL==2) {   
    ret_wert=ICFRQ_TIM2_Var[1].wert;
    ICFRQ_TIM2_Var[1].wert=0;
  } 
  if(ICFRQ_TIM2[vname].CHANNEL==3) {   
    ret_wert=ICFRQ_TIM2_Var[2].wert;
    ICFRQ_TIM2_Var[2].wert=0;
  } 
  if(ICFRQ_TIM2[vname].CHANNEL==4) {   
    ret_wert=ICFRQ_TIM2_Var[3].wert;
    ICFRQ_TIM2_Var[3].wert=0;
  } 

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// Init aller IO-Pins
//--------------------------------------------------------------
void P_ICFRQ_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  ICFRQ_TIM2_NAME_t vname;

  for(vname=0;vname<ICFRQ_TIM2_ANZ;vname++) {
    // Clock Enable
    RCC_AHB1PeriphClockCmd(ICFRQ_TIM2[vname].IC_CLK, ENABLE);

    // Config des Pins als AF-Input
    GPIO_InitStructure.GPIO_Pin = ICFRQ_TIM2[vname].IC_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(ICFRQ_TIM2[vname].IC_PORT, &GPIO_InitStructure);

    // Alternative-Funktion mit dem IO-Pin verbinden
    GPIO_PinAFConfig(ICFRQ_TIM2[vname].IC_PORT, ICFRQ_TIM2[vname].IC_SOURCE, GPIO_AF_TIM2);
  }  
}



//--------------------------------------------------------------
// interne Funktion
// Init vom Timer
//--------------------------------------------------------------
void P_ICFRQ_InitTIM(void)
{
  TIM_ICInitTypeDef  TIM_ICInitStructure;
  ICFRQ_TIM2_NAME_t vname;

  // Clock enable
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE); 

  // Vorteiler einstellen
  TIM_PrescalerConfig(TIM2, ICFRQ_TIM2_PRESCALE, TIM_PSCReloadMode_Immediate);

  for(vname=0;vname<ICFRQ_TIM2_ANZ;vname++) {
    if(ICFRQ_TIM2[vname].CHANNEL==1) {
      // Channel 1
      TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
      TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
      TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
      TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
      TIM_ICInitStructure.TIM_ICFilter = 0x0;
      TIM_ICInit(TIM2, &TIM_ICInitStructure);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==2) {
      // Channel 2
      TIM_ICInitStructure.TIM_Channel = TIM_Channel_2;
      TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
      TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
      TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
      TIM_ICInitStructure.TIM_ICFilter = 0x0;
      TIM_ICInit(TIM2, &TIM_ICInitStructure);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==3) {
      // Channel 3
      TIM_ICInitStructure.TIM_Channel = TIM_Channel_3;
      TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
      TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
      TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
      TIM_ICInitStructure.TIM_ICFilter = 0x0;
      TIM_ICInit(TIM2, &TIM_ICInitStructure);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==4) {
      // Channel 4
      TIM_ICInitStructure.TIM_Channel = TIM_Channel_4;
      TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising;
      TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
      TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;
      TIM_ICInitStructure.TIM_ICFilter = 0x0;
      TIM_ICInit(TIM2, &TIM_ICInitStructure);
    }
  }

  // Timer enable
  TIM_Cmd(TIM2, ENABLE);
}


//--------------------------------------------------------------
// interne Funktion
// Init vom NVIC
//--------------------------------------------------------------
void P_ICFRQ_InitNVIC(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;
  ICFRQ_TIM2_NAME_t vname;

  // NVIC init
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  for(vname=0;vname<ICFRQ_TIM2_ANZ;vname++) {
    if(ICFRQ_TIM2[vname].CHANNEL==1) {
      // Channel 1
      TIM_ITConfig(TIM2, TIM_IT_CC1, ENABLE);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==2) {
      // Channel 2
      TIM_ITConfig(TIM2, TIM_IT_CC2, ENABLE);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==3) {
      // Channel 3
      TIM_ITConfig(TIM2, TIM_IT_CC3, ENABLE);
    }
    if(ICFRQ_TIM2[vname].CHANNEL==4) {
      // Channel 4
      TIM_ITConfig(TIM2, TIM_IT_CC4, ENABLE);
    }
  }
}




//--------------------------------------------------------------
// ISR von Timer2
// wird bei einem Capture-Event aufgerufen
//--------------------------------------------------------------
void TIM2_IRQHandler(void)
{ 
  if(TIM_GetITStatus(TIM2, TIM_IT_CC1) == SET) {
    // Interrupt Flags loeschen
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC1);
    if(ICFRQ_TIM2_Var[0].flanke_nr == 0) {      
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[0].wert0 = TIM_GetCapture1(TIM2);

      ICFRQ_TIM2_Var[0].flanke_nr = 1;
    }
    else {
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[0].wert1 = TIM_GetCapture1(TIM2); 
      
      // differenz ausrechnen
      if(ICFRQ_TIM2_Var[0].wert1 > ICFRQ_TIM2_Var[0].wert0) {
        ICFRQ_TIM2_Var[0].wert = (ICFRQ_TIM2_Var[0].wert1 - ICFRQ_TIM2_Var[0].wert0); 
      }
      else {
        ICFRQ_TIM2_Var[0].wert = ((0xFFFF - ICFRQ_TIM2_Var[0].wert0) + ICFRQ_TIM2_Var[0].wert1); 
      }

      ICFRQ_TIM2_Var[0].flanke_nr = 0;
    }
  }

  if(TIM_GetITStatus(TIM2, TIM_IT_CC2) == SET) {
    // Interrupt Flags loeschen
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC2);
    if(ICFRQ_TIM2_Var[1].flanke_nr == 0) {      
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[1].wert0 = TIM_GetCapture2(TIM2);

      ICFRQ_TIM2_Var[1].flanke_nr = 1;
    }
    else {
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[1].wert1 = TIM_GetCapture2(TIM2); 
      
      // differenz ausrechnen
      if(ICFRQ_TIM2_Var[1].wert1 > ICFRQ_TIM2_Var[1].wert0) {
        ICFRQ_TIM2_Var[1].wert = (ICFRQ_TIM2_Var[1].wert1 - ICFRQ_TIM2_Var[1].wert0); 
      }
      else  {
        ICFRQ_TIM2_Var[1].wert = ((0xFFFF - ICFRQ_TIM2_Var[1].wert0) + ICFRQ_TIM2_Var[1].wert1); 
      }

      ICFRQ_TIM2_Var[1].flanke_nr = 0;
    }
  }

  if(TIM_GetITStatus(TIM2, TIM_IT_CC3) == SET) {
    // Interrupt Flags loeschen
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC3);
    if(ICFRQ_TIM2_Var[2].flanke_nr == 0) {      
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[2].wert0 = TIM_GetCapture3(TIM2);

      ICFRQ_TIM2_Var[2].flanke_nr = 1;
    }
    else {
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[2].wert1 = TIM_GetCapture3(TIM2); 
      
      // differenz ausrechnen
      if(ICFRQ_TIM2_Var[2].wert1 > ICFRQ_TIM2_Var[2].wert0) {
        ICFRQ_TIM2_Var[2].wert = (ICFRQ_TIM2_Var[2].wert1 - ICFRQ_TIM2_Var[2].wert0); 
      }
      else  {
        ICFRQ_TIM2_Var[2].wert = ((0xFFFF - ICFRQ_TIM2_Var[2].wert0) + ICFRQ_TIM2_Var[2].wert1); 
      }

      ICFRQ_TIM2_Var[2].flanke_nr = 0;
    }
  }

  if(TIM_GetITStatus(TIM2, TIM_IT_CC4) == SET) {
    // Interrupt Flags loeschen
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC4);
    if(ICFRQ_TIM2_Var[3].flanke_nr == 0) {      
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[3].wert0 = TIM_GetCapture4(TIM2);

      ICFRQ_TIM2_Var[3].flanke_nr = 1;
    }
    else {
      // aktuellen Wert auslesen
      ICFRQ_TIM2_Var[3].wert1 = TIM_GetCapture4(TIM2); 
      
      // differenz ausrechnen
      if(ICFRQ_TIM2_Var[3].wert1 > ICFRQ_TIM2_Var[3].wert0) {
        ICFRQ_TIM2_Var[3].wert = (ICFRQ_TIM2_Var[3].wert1 - ICFRQ_TIM2_Var[3].wert0); 
      }
      else {
        ICFRQ_TIM2_Var[3].wert = ((0xFFFF - ICFRQ_TIM2_Var[3].wert0) + ICFRQ_TIM2_Var[3].wert1); 
      }

      ICFRQ_TIM2_Var[3].flanke_nr = 0;
    }
  }
}
